'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var Register = _interopDefault(require('@vericus/slate-kit-utils-register-helpers'));
var immutable = require('immutable');
var isPlainObject = _interopDefault(require('is-plain-obj'));
var slate = require('slate');
var tinycolor = _interopDefault(require('tinycolor2'));
var tslib_1 = require('tslib');

var defaultOptions = {
    name: undefined,
    marks: undefined,
    type: undefined,
    styles: undefined,
    data: undefined,
    defaultColor: undefined,
    alpha: 1,
    renderer: undefined,
    label: undefined
};
var Options = /** @class */ (function (_super) {
    tslib_1.__extends(Options, _super);
    function Options() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Options.create = function (attrs) {
        if (attrs === void 0) { attrs = {}; }
        if (Options.isOptions(attrs))
            return attrs;
        if (isPlainObject(attrs))
            return Options.fromJSON(attrs);
        throw new Error("`Options.create` only accepts objects, strings or options");
    };
    Options.fromJSON = function (object) {
        var _a;
        if (Options.isOptions(object))
            return object;
        var name = object.name, type = object.type, data = object.data, styles = object.styles, defaultColor = object.defaultColor, _b = object.alpha, alpha = _b === void 0 ? 1 : _b, renderer = object.renderer;
        if (!type || !data || !styles || !defaultColor || !Array.isArray(styles)) {
            throw new Error("Highlight text require type, data and style option");
        }
        if (alpha < 0 || alpha > 1) {
            throw new Error("Alpha value can only go from 0 to 1");
        }
        return new Options({
            marks: (_a = {},
                _a[type] = type,
                _a),
            type: type,
            data: data,
            styles: styles,
            alpha: alpha,
            defaultColor: defaultColor,
            name: name,
            renderer: renderer
        });
    };
    Options.isOptions = function (args) {
        if (args instanceof immutable.Record) {
            var record_1 = args;
            return ["marks", "type", "data", "styles", "defaultColor"].every(function (key) {
                return record_1.has(key);
            });
        }
        return false;
    };
    return Options;
}(immutable.Record(defaultOptions, "highlight text option")));

var defaultStyle = {
    textDecoration: "inherit",
    textDecorationColor: "inherit"
};
function createProps(opts) {
    var type = opts.type, dataField = opts.data, styleFields = opts.styles, alpha = opts.alpha;
    return function (props) {
        if (!props.mark || props.mark.type !== type)
            return props;
        var attributes = props.attributes, data = props.mark.data;
        var style = (attributes || { style: {} }).style;
        var color = tinycolor(data.get(dataField));
        style = tslib_1.__assign({}, defaultStyle, style, { textDecoration: "inherit", decorationStyle: "initial" }, styleFields.reduce(function (acc, styleField) {
            var _a;
            return (tslib_1.__assign({}, acc, (_a = {}, _a[styleField] = color.setAlpha(alpha).toRgbString(), _a)));
        }, {}));
        return tslib_1.__assign({}, props, { attributes: tslib_1.__assign({}, props.attributes, { style: style }) });
    };
}

function createCommands(opt) {
    var _a;
    var defaultColor = opt.defaultColor, data = opt.data, type = opt.type, name = opt.name;
    return _a = {},
        _a["change" + name + "Color"] = function (editor, color) {
            var _a, _b;
            var value = editor.value;
            var selection = value.selection;
            if (editor.hasMark(value, type)) {
                if (tinycolor(color).toName() === defaultColor) {
                    if (selection.isCollapsed) {
                        editor.removeCollapsedMark(type);
                    }
                    else {
                        editor.removeExpandedMark(type);
                    }
                }
                else {
                    var newMark = new slate.Mark({
                        type: type,
                        data: slate.Data.create((_a = {},
                            _a[data] = color,
                            _a))
                    });
                    if (selection.isCollapsed) {
                        editor
                            .removeCollapsedMark(type)
                            .focus()
                            .addMark(newMark);
                    }
                    else {
                        editor.removeExpandedMark(type).addMarkAtRange(selection, newMark);
                    }
                }
            }
            else if (tinycolor(color).toName() !== defaultColor) {
                var mark = new slate.Mark({
                    type: type,
                    data: slate.Data.create((_b = {},
                        _b[data] = color,
                        _b))
                });
                if (selection.isCollapsed) {
                    editor.addMark(mark);
                }
                else {
                    editor.removeExpandedMark(type).addMarkAtRange(selection, mark);
                }
            }
            editor.focus();
            return;
        },
        _a;
}

function createUtils(opts) {
    var _a;
    var defaultColor = opts.defaultColor, data = opts.data, type = opts.type, name = opts.name;
    return _a = {},
        _a["current" + name + "Color"] = function (editor) {
            var value = editor.value;
            if (value.selection && value.selection.isFocused && value.activeMarks) {
                var activeColorMarks = value.activeMarks
                    .filter(function (mark) { return !!(mark && mark.type === type); })
                    .reduce(function (colors, mark) {
                    if (mark && mark.data.get(data)) {
                        return tslib_1.__spread(colors, [mark.data.get(data)]);
                    }
                    return colors;
                }, []);
                var uniqueColorMarks = tslib_1.__spread(Array.from(new Set(activeColorMarks)));
                if (uniqueColorMarks.length !== 0) {
                    return uniqueColorMarks[0];
                }
            }
            return defaultColor;
        },
        _a;
}

function createPlugin(pluginOptions) {
    if (pluginOptions === void 0) { pluginOptions = {}; }
    var options = Options.create(pluginOptions);
    var marks = options.marks, renderer = options.renderer;
    var commands = createCommands(options);
    var queries = createUtils(options);
    var props = createProps(options);
    var plugins = [
        Register({
            options: options,
            marks: marks,
            props: props
        }),
        {
            commands: commands,
            queries: queries
        }
    ];
    if (renderer) {
        var rendererPlugins = renderer(options);
        if (Array.isArray(rendererPlugins)) {
            plugins = tslib_1.__spread(plugins, rendererPlugins);
        }
        else {
            plugins = tslib_1.__spread(plugins, [rendererPlugins]);
        }
    }
    return plugins;
}

exports.default = createPlugin;
//# sourceMappingURL=slate-kit-highlight-text.js.map
